import org.kwis.msp.db.*;
import org.kwis.msp.lcdui.*;
import org.kwis.msp.lwc.*;

public class PhoneBook extends Jlet {

	private static final int DB_BUF_SIZE = 12;
	private static final byte NAME_DATA = (byte)'N';
	private static final byte PHONE_DATA = (byte)'P';
	
	DataBase db;
	byte[] dbBuf = new byte[DB_BUF_SIZE];
	DataFilter nameDf = new MyFilter(NAME_DATA);
	DataFilter phoneDf = new MyFilter(PHONE_DATA);
	DataComparatorInteger phoneDc = new DataComparatorInteger(0);

	ShellComponent shell;
	FormComponent form;

	FormComponent buttonForm;
	ButtonComponent inputButton;
	ButtonComponent removeButton;
	ButtonComponent nameShowButton;
	ButtonComponent phoneShowButton;
	ButtonComponent allShowButton;

	ShellComponent subShell;
	FormComponent subForm;
	TextFieldComponent nameTf;
	TextFieldComponent phoneTf;
	ListComponent fileList;
	ButtonComponent closeButton;

	ActionListener action = new ActionListener() {

		public void action(Component c, Object o) {
			if (c == closeButton) {
				if (subShell.isShown()){
					byte[] nameBuf = new byte[DB_BUF_SIZE];
					nameBuf[0] = NAME_DATA;
					byte[] data = nameTf.getString().getBytes();
					System.arraycopy(data, 0, nameBuf, 1, data.length);
		
					byte[] phoneBuf = new byte[DB_BUF_SIZE];
					phoneBuf[0] = PHONE_DATA;
					data = phoneTf.getString().getBytes();
					System.arraycopy(data, 0, phoneBuf, 1, data.length);
		
					try {
						db.insertRecord(nameBuf);
						db.insertRecord(phoneBuf);
					}catch(Exception e){}
					
					subShell.hide();
					
				}
			} else if (c == inputButton) {
				input();
			} else if (c == removeButton) {
				remove();
			} else if (c == nameShowButton) {
				list(0);
			} else if (c == phoneShowButton) {
				list(1);
			} else if (c == allShowButton) {
				list(2);
			} else if (c == fileList) {
				subShell.hide();
			}

		}
	};

	protected void startApp(String[] arg0) {

		try {
			db = DataBase.openDataBase("phonebook", DB_BUF_SIZE, true);
		} catch (IllegalArgumentException e) {
		} catch (DataBaseException e) {
		}

		mainFormCreate();
		subFormCreate();
		shellCreate();

		subShell.addComponent(subForm);
		shell.addComponent(form);

		shell.show();

	}
	
	protected void destroyApp(boolean arg0) {

	}
	
	private void shellCreate() {	
		shell = new ShellComponent();
		shell.setTitle("ȭȣ");

		subShell = new ShellComponent(10, 10, 100, 120);
		subShell.setBackground(0xffff00);
		subShell.setForeground(0xffffff);
	}

	private void mainFormCreate() {
		form = new FormComponent();
		inputButton = new ButtonComponent("Է", null);
		inputButton.setActionListener(action, null);
		removeButton = new ButtonComponent(" ", null);
		removeButton.setActionListener(action, null);
		nameShowButton = new ButtonComponent("̸ ", null);
		nameShowButton.setActionListener(action, null);
		phoneShowButton = new ButtonComponent("ȣ ", null);
		phoneShowButton.setActionListener(action, null);
		allShowButton = new ButtonComponent(" ", null);
		allShowButton.setActionListener(action, null);

		form.addComponent(inputButton);
		form.addComponent(removeButton);
		form.addComponent(nameShowButton);
		form.addComponent(phoneShowButton);
		form.addComponent(allShowButton);
	}

	private void subFormCreate() {
		subForm = new FormComponent();
		subForm.addComponent(new LabelComponent("̸"));
		nameTf = new TextFieldComponent("", TextComponent.CONSTRAINT_ANY);
		subForm.addComponent(nameTf);
		subForm.addComponent(new LabelComponent("ȭȣ"));
		phoneTf = new TextFieldComponent("", TextComponent.CONSTRAINT_NUMBER);
		subForm.addComponent(phoneTf);
		closeButton = new ButtonComponent("Է ݱ", null);
		closeButton.setActionListener(action, null);
		subForm.addComponent(closeButton);

		fileList = new ListComponent(ListComponent.SELECT_IMPLICIT);
		fileList.controlNumber(false);
		fileList.setActionListener(action, null);
	}

	private void input() {
		subShell.removeAllComponents();
		subShell.setTitle("ּ Է");
		subShell.addComponent(subForm);
		subShell.show();
	}
	private void remove() {
		try {
			db.closeDataBase();
			DataBase.deleteDataBase("phonebook");
			db = DataBase.openDataBase("phonebook", DB_BUF_SIZE, true);
		} catch (DataBaseException e) {
		}
	}

	private void list(int mode) {
		subShell.removeAllComponents();
		subShell.setTitle("ּҷ ");
		fileList.removeAllComponents();
		subShell.addComponent(fileList);
		switch (mode) {
			case 0 :
				try {
					int[] recordIds = db.sortRecord(nameDf, null);
					if(recordIds.length == 0) return;
					for(int i=0;i<recordIds.length;i++){
						byte[] data = db.selectRecord(i);
						ListItemComponent item = new ListItemComponent(new String(data, 1, data.length-1));
						item.setBackground(0xff0000);
						fileList.addComponent(item);
					}
				} catch (Exception e1) {}
				break;
			case 1 :
				try {
					int[] recordIds = db.sortRecord(phoneDf, phoneDc);
					if(recordIds.length == 0) return;
					for(int i=0;i<recordIds.length;i++){
						byte[] data = db.selectRecord(i);
						ListItemComponent item = new ListItemComponent(new String(data, 1, data.length-1));
						item.setBackground(0x00ff00);
						fileList.addComponent(item);
					}
				} catch (Exception e1) {}
				break;
			case 2 :
				if(db.getDataBaseSize() == 0) return;
				for(int i=0;i<db.getDataBaseSize();i++){
					try {
						byte[] data = db.selectRecord(i);
						ListItemComponent item = new ListItemComponent(new String(data, 1, data.length-1));
						if(data[0] == NAME_DATA){
							item.setBackground(0xff0000);
						}else{
							item.setBackground(0x00ff00);
						}
						fileList.addComponent(item);
					} catch (Exception e) {}
				}
				break;
		}
		subShell.show();
	}

	class MyFilter implements DataFilter{
		
		byte check;
		
		MyFilter(byte b){
			check = b;
		}

		public boolean filter(byte[] data) {
			if(check == data[0]) return true;
			return false;
		}
		
	}

}
